#include "copyright.h"
#include "error_msg.h"

//=========================================================================
// anyimg.cpp -  'any' file format implementation.
//
// Description:
//	This file implements 'any' format class read write operations
//
// RCS information:
// ----------------
// $RCSfile: anyimg.cpp,v $
// $Revision: 1.1 $
// $Date: 2002/01/29 20:20:22 $
//
//=========================================================================


#include <stdio.h>
//#include <malloc.h>
#include <stdlib.h>

// Add here envelope header for the new format.

#include "pnm_env.h"
#include "bmp_env.h"

// Do not change.
#include "anyimg.h"

format_t anyimg::cur_format = pnmFormat; 

//===============================
// Probe - probe file format
// If format is Unknown -
// type value is meeningless.
//================================

void anyimg::Probe(char *fname, format_t &format, data_t &type)
{
  FILE *fp;
  char header[1024];
  
  if ((fp = fopen(fname,"r")) == NULL){
    error_msg("anyimg::Probe(%s) failed", (int) fname);
    clip_exception(MALLOC);
  }
  
  fread(header,256,1,fp);
  fclose(fp);
  
  format = unknownFormat;
  
  // probe PGM
  if(!strncmp(header,"P5",2)){
    format	= pnmFormat;
    type	= bMonoType;
    return;
  }
  
  // probe PPM
  if(!strncmp(header,"P6",2)){
    format	= pnmFormat;
    type	= bRgbType;
    return;
  }
  // probe PFM
  if(!strncmp(header,"P7",2)){
    format	= pnmFormat;
    type	= fMonoType;
    return;
  }
	
  // probe PCM
  if(!strncmp(header,"P8",2)){
    format	= pnmFormat;
    type	= fRgbType;
    return;
  }
  
  // add here New format probing code.

  if(!strncmp(header,"BM",2)){
    format	= bmpFormat;
    type	= fRgbType;
    return;
  }
  
  
  return;
}



//===============================
// Read - read any format files
//================================

anyimg &anyimg::Read(char *fname)
{
  format_t	format;
  data_t	type;
  
  FreeCash();
  Probe(fname, format, type);
  
  if (format == unknownFormat){
	error_msg("anyimg::Read(%s): unsupported format\n", (int) fname);
    clip_exception(UNKNOWN_FORMAT);
  }
  
  cur_format = format;
  cur_type   = type;
  
  switch (cur_format + cur_type) {
    
  case pnmFormat + bMonoType :
    byte_mono_img = (bMonoImg) pnm(fname);
    cur_type = bMonoType;
    break;

  case pnmFormat + fMonoType  :		
    float_mono_img = (fMonoImg) pnm(fname);
    cur_type = fMonoType;
    break;
    
  case pnmFormat + bRgbType :
    byte_rgb_img = (bRgbImg) pnm(fname);
    cur_type = bRgbType;
    break;

  case pnmFormat + fRgbType  :		
    float_rgb_img = (fRgbImg) pnm(fname);
    cur_type = fRgbType;
    break;
    
    
  case bmpFormat + bRgbType :
    byte_rgb_img = (bRgbImg) bmp(fname);
    cur_type = bRgbType;
    break;

  case bmpFormat + fRgbType :
    float_rgb_img = (fRgbImg) bmp(fname);
    cur_type = fRgbType;
    break;

    // Add New format and types here.
    
  default:
    assert(("switch default",0));
  }
  return *this;
}




//================================
// Write - write 'any format image
//================================


void anyimg::Write(char *fname)
{
  char fullname[256];
  
  strcpy(fullname,fname);
  SetExt(fullname,cur_format, cur_type);
  
  switch (cur_format + cur_type) {

  case pnmFormat + bMonoType  :
    pnm(byte_mono_img).Write(fullname);
    break;
    
  case pnmFormat + fMonoType :
    pnm(float_mono_img).Write(fullname);
    break;
    
  case pnmFormat + bRgbType  :
    pnm(byte_rgb_img).Write(fullname);
    break;
    
  case pnmFormat + fRgbType :
    pnm(float_rgb_img).Write(fullname);
    break;
    
  case bmpFormat + bRgbType :
    bmp(byte_rgb_img).Write(fullname);
    break;
    
  case bmpFormat + fRgbType :
    bmp(float_rgb_img).Write(fullname);
    break;

    //add new format and types here.
    
  default:
    assert(("switch default",0));
  }
}


//==================================
// SetExt - Set file name extention
//==================================

void anyimg::SetExt(char *fname, format_t format, data_t type)
{
  char *dotp;
  
  dotp = strrchr(fname,'.');
  
  if (dotp != NULL) *dotp = '\0';
  
  switch (format + type) {
    
  case pnmFormat + bMonoType :
    strcat(fname,".pgm");
    break;
    
  case pnmFormat + fMonoType :
    strcat(fname,".pfm");
    break;
    
  case pnmFormat + bRgbType :
    strcat(fname,".ppm");
    break;
	
  case pnmFormat + fRgbType :
    strcat(fname,".pcm");
    break;

  case bmpFormat + bRgbType :
  case bmpFormat + fRgbType :
    strcat(fname,".bmp");
    break;
    
    // add new format and type here.
    
  default:
    assert(("switch defualt",0));
  }
}







