#include "copyright.h"

//=================================================================
// anyimg.h -  'any' file format header file.
//
// Description:
//    This class implements generic image I/O class.
//    operations that are not allowed for a particular class will raise
//    an exception.
//
//    Images are cashed by their data-type and not by their format - this
//    enable possible change of format after an image was read or assigned. 
//	  The user can alters the format family not the data type.
//
//    rules:
//          0. class format is uvinersal.
//          1. default format is Pnm.
//          2. class format is changed by a Read call to the last read
//             file format.
//          3. class format is changed by an explicit SetFormat call
//			4. file extention in Write calles are set by the class.
//          5. file extentions in Read calls are ignored.
//          
//               
//	  This file contains instruction for adding new file formats.
//
//
// RCS information:
// ----------------
// $RCSfile: anyimg.h,v $
// $Revision: 1.2 $
// $Date: 1997/03/12 18:04:34 $
//
//=================================================================


#ifndef ANYIMG_ENV_H
#define ANYIMG_ENV_H

//do not change the following lines.
#include "image.h"

// Add here include line for the envelope file of the new format
#include "pnm_env.h"
#include "bmp_env.h"

// add here the name and description of the new format

typedef enum {   
  unknownFormat = 0,	
  pnmFormat     = 1,	// pgm ppm [pfm pcm]-private
  bmpFormat	= 2	// microsoft bitmap format
} format_t;
  
  
// Internal Image data types - do not change.

typedef enum { 
  fMonoType   = 100,		
  fRgbType    = 200,		
  bMonoType   = 300,
  bRgbType    = 400,		
} data_t;



class anyimg {
  
  // Current format setting (shared).
  // do not change.
  static format_t anyimg::cur_format;	
  
  // Current data type setting
  // do not change.
  data_t  cur_type;
  
  
  // image cash by *data type*
  // do not change these lines
  
  fMonoImg  float_mono_img;
  fRgbImg   float_rgb_img;
  bMonoImg  byte_mono_img;
  bRgbImg   byte_rgb_img;
  
  // private functions
  
  void Probe(char *fname, format_t &format, data_t &type);
  void SetExt(char *fname, format_t format, data_t Type);	
  
  void FreeCash(){
    float_mono_img = fMonoImg();
    float_rgb_img  = fRgbImg();
    byte_mono_img  = bMonoImg();
    byte_rgb_img   = bRgbImg();
  }
  
public:
  
  void SetFormat(format_t fmt){
    cur_format = fmt;
  }
  
  // Generic I/O operations.
  // do not change.
  anyimg &Read(char *fname);
  void Write(char *fname);
  
  
  // Assignmnet operator for each data type.
  // No support for Cie.
  // do not change
  
  anyimg &operator= (fMonoImg &img) {
    FreeCash();
    float_mono_img = img;
    cur_type = fMonoType;
    return *this;
  }
  
  
  anyimg &operator= (fRgbImg &img) {
    FreeCash();
    float_rgb_img = img;
    cur_type = fRgbType;
    return *this;
  }
  
  // byte images

  anyimg &operator= (bMonoImg &img) {
    FreeCash();
    byte_mono_img = img;
    cur_type = bMonoType;
    return *this;
  }
  
  
  anyimg &operator= (bRgbImg &img) {
    FreeCash();
    byte_rgb_img = img;
    cur_type = bRgbType;
    return *this;
  }
  
  
  // Casting operator into the proper data type for the new class

  operator fMonoImg(){

    switch(cur_type){
    case fMonoType:
      return  float_mono_img; 
      break;
      
    case fRgbType:
      return  (fMonoImg) float_rgb_img;
      break;	

    case bMonoType:
      return  (fMonoImg) byte_mono_img; 
      break;
      
    case bRgbType:
      return  (fMonoImg) byte_rgb_img;
      break;	
    default:
      assert(("default switch",0));
      return fMonoImg();                // satisfy MSVC
    }
  }
  
  
  
  operator fRgbImg(){
    
    switch(cur_type){
    case fMonoType:
      return  (fRgbImg) float_mono_img;
      break;	
      
    case fRgbType:
      return  float_rgb_img; 
      break;	

    case bMonoType:
      return  (fRgbImg) byte_mono_img;
      break;	
      
    case bRgbType:
      return  (fRgbImg) byte_rgb_img;
      break;	
      
    default:
      assert(("default switch",0));
      return fRgbImg();                 // satisfy MSVC
    }
  }

  // byte images

  operator bMonoImg(){

    switch(cur_type){
    case fMonoType:
      return  (bMonoImg) float_mono_img; 
      break;
      
    case fRgbType:
      return  (bMonoImg) float_rgb_img;
      break;	

    case bMonoType:
      return  byte_mono_img;
      break;
      
    case bRgbType:
      return  (bMonoImg) byte_rgb_img;
      break;	

    default:
      assert(("default switch",0));
      return fMonoImg();                // satisfy MSVC
    }
  }
  
  
  
  operator bRgbImg(){
    
    switch(cur_type){
    case fMonoType:
      return  (bRgbImg) float_mono_img;
      break;	
      
    case fRgbType:
      return  (bRgbImg) float_rgb_img;
      break;	

    case bMonoType:
      return  (bRgbImg) byte_mono_img;
      break;	
      
    case bRgbType:
      return  byte_rgb_img; 
      break;	
    default:
      assert(("default switch",0));
      return fRgbImg();                 // satisfy MSVC
    }
  }


  operator fCieImg(){
    
    switch(cur_type){
    case fMonoType:
      return  (fCieImg) float_mono_img;
      break;	
      
    case fRgbType:
      return  (fCieImg) float_rgb_img;
      break;	
 
    case bMonoType:
      return  (fCieImg) byte_mono_img;
      break;	
      
    case bRgbType:
      return  (fCieImg) byte_rgb_img;
      break;	
      
    default:
      assert(("default switch",0));
      return fCieImg();                 // satisfy MSVC
    }
  }
  
  
  // constructors
  
  anyimg(char *fname) {
    Read(fname); 
  }
  
  
  // constructors for all image data types.
  // do not change
  
  anyimg(const fMonoImg &img){
    cur_type = fMonoType;
    float_mono_img = img;
  }
  
  anyimg(const fRgbImg &img){
    cur_type = fRgbType;
    float_rgb_img = img;
  }
  
  // byte images - used for exporting only.
  
  anyimg(const bMonoImg &img){
    cur_type = bMonoType;
    byte_mono_img = img;
  }
  
  anyimg(const bRgbImg &img){ 
    cur_type = bRgbType;
    byte_rgb_img = img;
  }
  
  
  // use default distructor
  
};




#endif

