#include "copyright.h"

/*=================================================================*
 * bmp.c - bmp rutines
 *
 * Description:
 *   Creat, Free, Read, Write BMP file format.
 *
 * RCS information:
 * ----------------
 * $RCSfile: bmp.cpp,v $
 * $Revision: 1.1 $
 * $Date: 2002/01/29 20:20:22 $
 *
 * Author: Ziv Yaniv email: zivy@cs.huji.ac.il
 *
 *=================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include "bmp.h"

#define MAX_SIZE 4
#define BM 19778

#define get_uchar(fp) (getc(fp))
#define put_uchar(c,fp) (putc(c,fp))

unsigned short get_ushort(FILE *fp);
long get_long(FILE *fp);
unsigned long get_ulong(FILE *fp);
short load_header(FILE *input,BmpFilePtr bfptr);
short validate_header(BmpFileHeaderPtr bfhp);
short load_info(FILE *input,BmpFilePtr bfptr);
short validate_info(BmpInfoHeaderPtr bihp);
void set_cmap_entry1(ColorMapPtr cmp,int ent,RgbQuadPtr q);
void set_cmap_entry2(ColorMapPtr cmp,int ent,unsigned char r,
		     unsigned char g,unsigned char b);
RgbQuadPtr get_cmap_entry(ColorMapPtr cmp,int ent);
void print_cmap(ColorMapPtr cmp);
short load_cmap(FILE *input,ColorMapPtr cmp);

short set_entry(BmpFilePtr bfptr,int i,int j,RgbQuadPtr rgbq);
short load_bmp24(FILE *input, BmpFilePtr bfptr); 
short load_bmp1(FILE *input, BmpFilePtr bfptr); 
short load_bmp4(FILE *input, BmpFilePtr bfptr); 
short load_bmp8(FILE *input, BmpFilePtr bfptr); 


/************************************************************************/
/*
 * This function reads an unsigned long (4 bytes) from least signifcant
 * byte to most significant.
 */
unsigned long get_ulong(FILE *fp)
{
  unsigned char c[MAX_SIZE];
  int shft,i,size=sizeof(unsigned long);
  unsigned long value=0;

  for(i=0;i<size;i++)
    c[i]=getc(fp);
  for(i=0;i<size;i++) {
    shft=i*8;
    value+=((unsigned long)c[i])<<shft;
  }
  return(value);
}
/************************************************************************/
/*
 * This function writes an unsigned long (4 bytes) from least signifcant
 * byte to most significant.
 */
void put_ulong(unsigned long l,FILE *fp)
{
  unsigned char c[MAX_SIZE];
  int shft,i,size=sizeof(unsigned long);

  for(i=0;i<size;i++) {
    shft=i*8;
    c[i]=(unsigned char)((l>>shft) & 0xff);
  }
  for(i=0;i<size;i++)
    putc(c[i],fp);
}
/************************************************************************/
long get_long(FILE *fp)
{
  unsigned char c[MAX_SIZE];
  int shft,i,size=sizeof(long);
  unsigned long value=0;

  for(i=0;i<size;i++)
    c[i]=getc(fp);
  for(i=0;i<size;i++) {
    shft=i*8;
    value+=((long)c[i])<<shft;
  }
  return(value);
}
/************************************************************************/
void put_long(long l,FILE *fp)
{
  unsigned char c[MAX_SIZE];
  int shft,i,size=sizeof(long);

  for(i=0;i<size;i++) {
    shft=i*8;
    c[i]=(unsigned char)((l>>shft) & 0xff);
  }
  for(i=0;i<size;i++)
    putc(c[i],fp);
}
/************************************************************************/
unsigned short get_ushort(FILE *fp)
{
  unsigned char c[MAX_SIZE];
  int shft,i,size=sizeof(unsigned short);
  unsigned short value=0;

  for(i=0;i<size;i++)
    c[i]=getc(fp);
  for(i=0;i<size;i++) {
    shft=i*8;
    value+=((unsigned short)c[i])<<shft;
  }
  return(value);
}
/************************************************************************/
void put_ushort(unsigned short s,FILE *fp)
{
  unsigned char c[MAX_SIZE];
  int shft,i,size=sizeof(unsigned short);

  for(i=0;i<size;i++) {
    shft=i*8;
    c[i]= (unsigned char) ((s >> shft) & 0xff);
  }
  for(i=0;i<size;i++)
    putc(c[i],fp);

}
/************************************************************************/
/*
 *This function loads the bitmap file header from the given file and checks
 *that it is valid. 
 * Parameters: input - open bitmap file.
 *             bfptr - pointer to bitmap data struct.
 * Return values:  0 Succes.
 *                -1 Failure.
 * Bitmap header specification:
 *
 *   unsigned short   bfType;
 *   unsigned long    bfSize;
 *   unsigned short   bfReserved1;
 *   unsigned short   bfReserved2;
 *   unsigned long    bfOffBits;
 *
 *   Member          Description
 *
 *   bfType          Specifies the type of file. This member must be BM. 
 *   bfSize          Specifies the size of the file, in bytes. 
 *   bfReserved1     Reserved; must be set to zero. 
 *   bfReserved2     Reserved; must be set to zero.
 *   bfOffBits       Specifies the byte offset from the BITMAPFILEHEADER 
 *                   structure to the actual bitmap data in the file.
 */

short load_header(FILE *input,BmpFilePtr bfptr) 
{
  bfptr->bfh.bftype=get_ushort(input);
  if(feof(input))
    return(-1);

  bfptr->bfh.bfsize=get_ulong(input);   /*get size of file*/
  if(feof(input))
    return(-1);

  bfptr->bfh.bfreserved1=get_ushort(input); /*get reserved1*/
  if(feof(input))
    return(-1);

  bfptr->bfh.bfreserved2=get_ushort(input);  /*get reserved2*/
  if(feof(input))
    return(-1);
  
  bfptr->bfh.bfoffbits=get_ulong(input);  /*get offset to bitmap*/
  if(feof(input))
    return(-1);
                             /*check header validty*/
  if(validate_header(&(bfptr->bfh))==-1)
    return(-1);
  return(0);
}
/************************************************************************/
/*
 * This function checks that the given bmp file header is valid ,see 
 * function load_header for more info.
 * Return values: 0 Success.
 *               -1 Failure.
 */
short validate_header(BmpFileHeaderPtr bfhp)
{
  if(bfhp->bftype!=BM)  /*not bitmap file*/
    return(-1);
                            /*reserved must be set to 0*/
  if((bfhp->bfreserved1!=0) || (bfhp->bfreserved2!=0))
    return(-1);

  return(0);

}
/************************************************************************/
/*
 *This function loads the bitmap info header from the given file and checks
 *that it is valid. 
 * Parameters: input - open bitmap file.
 *             bfptr - pointer to bitmap data struct.
 * Return values:  0 Succes.
 *                -1 Failure.
 * Bitmap info specification:
 *   unsigned long    biSize;
 *   long             biWidth;
 *   long             biHeight;
 *   unsigned short   biPlanes;
 *   unsigned short   biBitCount;
 *   unsigned long    biCompression;
 *   unsigned long    biSizeImage;
 *   long             biXPelsPerMeter;
 *   long             biYPelsPerMeter;
 *   unsigned long    biClrUsed;
 *   unsigned long    biClrImportant;
 *
 *   Member          Description
 *
 *   biSize          Specifies the number of bytes required by the
 *                   BITMAPINFOHEADER structure.
 *   biWidth         Specifies the width of the bitmap, in pixels. 
 *   biHeight        Specifies the height of the bitmap, in pixels. 
 *   biPlanes        Specifies the number of planes for the target device. 
 *                   This member must be set to 1.
 *   biBitCount      Specifies the number of bits per pixel. This value must 
 *                   be 1,4,8,or 24.
 *   biCompression   Specifies the type of compression for a compressed 
 *                   bitmap. It can be one of the following values:
 *                               BI_RGB   Specifies that the bitmap is not 
 *                                        compressed.
 *                               BI_RLE8  Specifies a run-length encoded 
 *                                        format for bitmaps with 8 bits
 *                                        per pixel. The compression format is
 *                                        a 2-byte format consisting of a count
 *                                        byte followed by a byte containing a
 *                                        color index.
 *                               BI_RLE4  Specifies a run-length encoded 
 *                                        format for bitmaps with 4 bits per 
 *                                        pixel. The compression format is a 
 *                                        2-byte format consisting of a count
 *                                        byte followed by two word-length 
 *                                        color indexes.
 *   biSizeImage     Specifies the size, in bytes, of the image. It is valid to
 *                   set this member to zero if the bitmap is in the BI_RGB 
 *                   format.
 *   biXPelsPerMeter Specifies the horizontal resolution, in pixels per meter,
 *                   of the target device for the bitmap.
 *   biYPelsPerMeter Specifies the vertical resolution, in pixels per meter, of
 *                   the target device for the bitmap.
 *   biClrUsed       Specifies the number of color indexes in the color table
 *                   actually used by the bitmap. If this value is zero, the 
 *                   bitmap uses the maximum number of colors corresponding to
 *                   the value of the biBitCount member.
 *   biClrImportant  Specifies the number of color indexes that are considered
 *                   important for displaying the bitmap. If this value is 
 *                   zero, all colors are important.
 */
short load_info(FILE *input,BmpFilePtr bfptr)
{
  bfptr->bfih.bisize=get_ulong(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.biwidth=get_long(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.biheight=get_long(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.biplanes=get_ushort(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.bibitcount=get_ushort(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.bicompression=get_ulong(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.bisizeimage=get_ulong(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.bixpelspermeter=get_long(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.biypelspermeter=get_long(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.biclrused=get_ulong(input);
  if(feof(input))
    return(-1);

  bfptr->bfih.biclrimportant=get_ulong(input);
  if(feof(input))
    return(-1);
  
                             /*check info validty*/
  if(validate_info(&(bfptr->bfih))==-1)
    return(-1);

  return(0);
  
}
/************************************************************************/
/*
 * This function checks that the given bmp info header is valid ,see 
 * function load_info for more info :) .
 * Return values: 0 Success.
 *               -1 Failure.
 */
short validate_info(BmpInfoHeaderPtr bihp)
{
    if((bihp->bibitcount!=1 && bihp->bibitcount!=4 && bihp->bibitcount!=8 && 
      bihp->bibitcount!=24) || (bihp->biplanes!=1) || 
     ((bihp->bicompression!=BI_RGB) && (bihp->bicompression!=BI_RLE8) &&
      (bihp->bicompression!=BI_RLE4)))
      return(-1);
    
    if(((bihp->bibitcount==1 || bihp->bibitcount==24) &&
	(bihp->bicompression!=BI_RGB)) ||
       (bihp->bibitcount==4 && (bihp->bicompression==BI_RLE8)) ||
       (bihp->bibitcount==8 && (bihp->bicompression==BI_RLE4)))
      return(-1);

    return(0);
}
  

/************************************************************************/
/*
 * This procedure sets the entry in the color map according to the given
 * RgbQuad q.
 */
void set_cmap_entry1(ColorMapPtr cmp,int ent,RgbQuadPtr q)
{
  if(ent>=0 && ent<cmp->size) {
    cmp->table[ent].blue=q->blue;
    cmp->table[ent].red=q->red;
    cmp->table[ent].green=q->green;
    cmp->table[ent].reserved=q->reserved;
  }
}
/**************************************************************************/  
/*
 * This procedure sets the entry in the color map according to the given
 * colors r,g,b.
 */
void set_cmap_entry2(ColorMapPtr cmp,int ent,unsigned char r,
		     unsigned char g,unsigned char b)
{
  if(ent>=0 && ent<cmp->size) {
    cmp->table[ent].blue=b;
    cmp->table[ent].red=r;
    cmp->table[ent].green=g;
    cmp->table[ent].reserved=0;
  }
}
/**************************************************************************/  
/*
 * This function returns a pointer to the ent rgb quad struct in the colormap.
 * Return values: RgbQuadPtr to entry Success.
 *                NULL                Failure (ent is out of bounds).
 */
 
RgbQuadPtr get_cmap_entry(ColorMapPtr cmp,int ent)
{
  if((ent>=0) && (ent<cmp->size)) 
    return(&(cmp->table[ent]));
  return(NULL);
}
/**************************************************************************/  
/*
 * This function prints (to stdout) the values of the given color map.
 */
void print_cmap(ColorMapPtr cmp) 
{
  int i;
  RgbQuadPtr ent;

  for(i=0;i<cmp->size;i++) {
    ent=get_cmap_entry(cmp,i);
    printf("%u %u %u\n",ent->red,ent->green,ent->blue,ent->reserved);
  } 
}
/**************************************************************************/  
/*
 * This function loads a color map from a given file where each 4 bytes  
 * represent a color r,g,b & reserved.
 */
short load_cmap(FILE *input,ColorMapPtr cmp)
{
  int i;

  for(i=0;i<cmp->size;i++) {
    cmp->table[i].blue=get_uchar(input);
    if(feof(input))
      return(-1);

    cmp->table[i].green=get_uchar(input);
    if(feof(input))
      return(-1);

    cmp->table[i].red=get_uchar(input);
    if(feof(input))
      return(-1);

    cmp->table[i].reserved=get_uchar(input);
    if(feof(input))
      return(-1);
  }
  return(0);
}
/**************************************************************************/  
/*
 *This function loads the bitmap file from the given file. 
 * Parameters: fname - name of bitmap file.
 *             bfptr - pointer to bitmap data struct.
 * Return values:  0 Succes.
 *                -1 Failure.
 */

short load_bmp_file(char *fname,BmpFilePtr bfptr)
{
  FILE *input;

  if((input=fopen(fname,"rb"))==NULL) /*failed opening input file*/
    return(-1);
  if(load_bmp_file_stream(input,bfptr)==-1) {
    fclose(input);
    return(-1);
  }
  fclose(input);
  return(0);
}
/**************************************************************************/  
/*
 *This function loads the bitmap file from the given stream. 
 * Parameters: fname - name of bitmap file.
 *             bfptr - pointer to bitmap data struct.
 * Return values:  0 Succes.
 *                -1 Failure.
 */
short load_bmp_file_stream(FILE *input,BmpFilePtr bfptr)
{
  int i,cmaplen,len;
  long height;
  long width;
  
  if(load_header(input,bfptr)==-1)  /*failed reading file header*/
    return(-1);
  if(load_info(input,bfptr)==-1)  /*failed reading info header*/ 
    return(-1);
  if(bfptr->bfih.bibitcount!=24){   /*we have a color map , get it*/
    len=bfptr->bfih.biclrused;
                  /*if the number of colors used is 0 this means all colors*/
                 /*are used so we have a color map of size 2^bitcount.*/
    cmaplen=(len ? len : 1<<bfptr->bfih.bibitcount);
    bfptr->cmap=(ColorMapPtr)calloc(1,sizeof(ColorMap));
    bfptr->cmap->size=cmaplen;
    bfptr->cmap->table=(RgbQuadPtr)calloc(cmaplen,sizeof(RgbQuad));
    if(load_cmap(input,bfptr->cmap)==-1) 
      return(-1);
  }

                          /*create the actual bitmap */
  height=bfptr->bfih.biheight;
  width=bfptr->bfih.biwidth;

  bfptr->image=(RgbPtr *)calloc(height,sizeof(RgbPtr));
  if(bfptr->image==NULL) 
    return(-1);
  for(i=0;i<height;i++) {
    bfptr->image[i]=(RgbPtr)calloc(width,sizeof(RgbRec));
    if(bfptr->image[i]==NULL) 
      return(-1);
  }
                /*set the bitmaps r g b values according to bitamp file*/
  switch(bfptr->bfih.bibitcount) {
  case 1:
    if(load_bmp1(input,bfptr)==-1)
      return(-1);
    break;
  case 4:
    if(load_bmp4(input,bfptr)==-1)
      return(-1);
    break;
  case 8:
    if(load_bmp8(input,bfptr)==-1)
      return(-1);
    break;
  case 24:
    if(load_bmp24(input,bfptr)==-1)
      return(-1);
    break;
  }
  return(0);
}
/************************************************************************/
/*
 * This function loads a 24 bit bitmap, every three bytes represent a color,
 * R,G,B.
 * Note: all lines in the image must end on a 32 bit boundry.
 * Return values: 0 Success.
 *               -1 Failure.
 */
short load_bmp24(FILE *input, BmpFilePtr bfptr) 
{
  int i,j,padb;
  long width=bfptr->bfih.biwidth;
  long height=bfptr->bfih.biheight;
  
  /*number of pad bytes at end of row so rows end on 32 bit boundry*/
  padb=(4-((width*3)%4)) & 0x03;
  
  for(i=height-1;i>=0;i--) {
    for(j=0;j<width;j++) {

      bfptr->image[i][j].b=get_uchar(input);
      if(feof(input)) 
	return(-1);

      bfptr->image[i][j].g=get_uchar(input);
      if(feof(input))
	return(-1);
      
      bfptr->image[i][j].r=get_uchar(input);
      if(feof(input))
	return(-1);
    }
    for(j=0;j<padb;j++) {
     get_uchar(input);
      if(feof(input))
	return(-1);
    }
  }
  return(0);
}
/************************************************************************/
/*
 * This function sets the i,j entry of the bitmap to the color given in the
 * rgb quad.
 * Return values: 0 Success.
 *               -1 Failure.
 */
short set_entry(BmpFilePtr bfptr,int i,int j,RgbQuadPtr rgbq)
{
  if((i<bfptr->bfih.biheight) && (i>=0) && (j<bfptr->bfih.biwidth) && (j>=0)
     && (rgbq!=NULL)) {
    bfptr->image[i][j].r=rgbq->red;
    bfptr->image[i][j].g=rgbq->green;
    bfptr->image[i][j].b=rgbq->blue;
    return(0);
  }
  return(-1);
}  
/************************************************************************/
/*
 * This function loads a binary bitmap, every bit represent a color in the
 * color map (entry 0 or entry 1).
 * 
 * Note: all lines in the image must end on a 32 bit boundry.
 * Return values: 0 Success.
 *               -1 Failure.
 */

short load_bmp1(FILE *input, BmpFilePtr bfptr) 
{
  long width=bfptr->bfih.biwidth;
  long height=bfptr->bfih.biheight;
  int i,j,bitnum,padw,index;
  unsigned char c;
  RgbQuadPtr ent;

                    /*we have to read 'padw' bits as each line must end */
                   /*on a 32 bit boundry*/
  padw=width+((32-(width%32)) & 0x1f);
  for(i=height-1;i>=0;i--) {
    for(j=bitnum=0;j<padw;j++,bitnum++) {
      if((bitnum&7)==0) {
        c=get_uchar(input);
	if(feof(input))
	  return(-1);
        bitnum=0;
      }
      if(j<width) {
        index= (c & 0x80) ? 1:0;
	ent=get_cmap_entry(bfptr->cmap,index);
	if(set_entry(bfptr,i,j,ent)==-1)
	  return(-1);
        c=c<<1;
      }
    }
  }
  return(0);
}
/************************************************************************/
/*
 * This function returns a pointer to the i,j rgb struct in the bitmap.
 * Return values: RgbPtr to entry Success.
 *                NULL            Failure (i,j are out of bounds).
 */
RgbPtr bmp_entry(BmpFilePtr bfptr,int i,int j)
{
  if((i<bfptr->bfih.biheight) && (i>=0) && (j<bfptr->bfih.biwidth) && (j>=0))
    return(&(bfptr->image[i][j]));
  return(NULL);
}
/************************************************************************/
/*
 * This function loads a 16 color bitmap, every 4 bits represent a color in the
 * color map (entry 0 - entry 15).
 * 
 * The bitmap may be in compressed mode (BI_RLE4) or not compressed (BI_RGB).
 * 
 * Encoded Mode
 *
 *  A unit of information in encoded mode consists of two bytes. The first byte
 *  of the pair contains the number of pixels to be drawn using the color 
 *  indexes in the second byte.
 *
 *  The second byte contains two color indexes, one in its high-order nibble
 *  (that is, its low-order 4 bits) and one in its low-order nibble.
 *
 *  The first pixel is drawn using the color specified by the high-order 
 *  nibble, the second is drawn using the color in the low-order nibble, the 
 *  third is drawn with the color in the high-order nibble, and so on, until 
 *  all the pixels specified by the first byte have been drawn.
 *  The first byte of the pair can be set to zero to indicate an escape that
 *  denotes the end of a line, the end of the bitmap, or a delta.
 *  Second byte     Meaning
 *
 *        0       End of line. 
 *        1       End of bitmap. 
 *        2       Delta. The two bytes following the escape contain unsigned 
 *                values indicating the horizontal and vertical offsets of the
 *                next pixel from the current position.
 * Absolute Mode
 *
 *  In absolute mode, the first byte contains zero, the second byte contains 
 *  the number of color indexes that follow, and subsequent bytes contain color
 *  indexes in their high- and low-order nibbles, one color index for each 
 *  pixel. Each run must be aligned on a word boundary.
 *
 * Note: all lines in the image must end on a 32 bit boundry.
 * Return values: 0 Success.
 *               -1 Failure.
 */

short load_bmp4(FILE *input, BmpFilePtr bfptr) 
{
  int i,j,k,index,padw,bitw;
  long width=bfptr->bfih.biwidth;
  long height=bfptr->bfih.biheight;
  unsigned char c,c1;
  RgbQuadPtr ent;

                                            
  if(bfptr->bfih.bicompression==BI_RGB) { /*file is not compressed.*/
    bitw=width*4; /*number of bits per valid row*/
             /*we have to read 'padw' quads (4 bits) as each line must end */
            /*on a 32 bit boundry  */
    padw=(bitw+((32-(bitw%32)) & 0x1f))>>2;
    for(i=height-1;i>=0;i--) {
      for(j=0;j<padw;j++) {
	if((j%2)==0) {  /*every 2 quads read a new byte*/
	  c=get_uchar(input);
	  if(feof(input))
	    return(-1);
	}
	if(j<width) {
	  index= (c & 0xf0)>>4;
	  ent=get_cmap_entry(bfptr->cmap,index);
	  if(set_entry(bfptr,i,j,ent)==-1)
	    return(-1);
	  c=c<<4;
	}
      }
    }
  }
  else if(bfptr->bfih.bicompression==BI_RLE4) { /*file is compressed*/
    j=0;
    i=height-1;
    while(i>=0) {
      c=get_uchar(input);
      if(feof(input))
	return(-1);
      if(c!=0x00) {
	c1=get_uchar(input);
	if(feof(input))
	  return(-1);
	for(k=0;k<c;k++,j++) {
	  index= (k&1) ? (c1 & 0x0f) :((c1>>4) & 0x0f);
	  ent=get_cmap_entry(bfptr->cmap,index);
	  if(set_entry(bfptr,i,j,ent)==-1)
	    return(-1);
	}
      }
                                /*entering escape sequences*/
      else {   
	c=get_uchar(input);
	if(feof(input))
	  return(-1);
	if(c==0x00) {   /*end of line*/
	  j=0;
	  i--;
	}
        else if(c==0x01) return(0);  /*end of bitmap*/
	         /*next 2 bytes contain horizontal and vertical offset of */
	        /*next pixel*/
	else if(c==0x02) {  
	  c=get_uchar(input);
	  if(feof(input))
	    return(-1);
	  j+=c;
	  c=get_uchar(input);
	  if(feof(input))
	    return(-1);
	  i+=c;
	}
	                     /*entering absolute mode 0xff=> c => 0x03 */
	else {  
	  for(k=0;k<c;k++,j++) {
	    if((k&1)==0) {
	      c1=get_uchar(input);
	      if(feof(input))
		return(-1);
	    }
	    index= (k&1) ? (c1 & 0x0f) :((c1>>4) & 0x0f);
	    ent=get_cmap_entry(bfptr->cmap,index);
	    if(set_entry(bfptr,i,j,ent)==-1)
	      return(-1);
	  }
	  if(((c&3)==1) || ((c&3)==2))  /*read pad byte*/
	    c=get_uchar(input);
	}
      }
    }
  }
  return(0);
}

/************************************************************************/
short load_bmp8(FILE *input, BmpFilePtr bfptr) 
{
  int padw,i,j,k;
  long width=bfptr->bfih.biwidth;
  long height=bfptr->bfih.biheight;
  unsigned char c,c1;
  RgbQuadPtr ent;

  if(bfptr->bfih.bicompression==BI_RGB) { /*file is not compressed.*/
         /*number of bytes we have to read so that rows end on 32 bit*/
    padw=((width+3)/4)*4;
    for(i=height-1;i>=0;i--)
      for(j=0;j<padw;j++) {
	c=get_uchar(input);
	if(j<width) {
	  ent=get_cmap_entry(bfptr->cmap,(int)c);
	  if(set_entry(bfptr,i,j,ent)==-1)
	    return(-1);
	}
      }
  }
  else if(bfptr->bfih.bicompression==BI_RLE8) { /*file is compressed*/
    j=0;
    i=height-1;
    while(i>=0) {
      c=get_uchar(input);
      if(feof(input))
	return(-1);
      if(c!=0x00) {
	c1=get_uchar(input);
	if(feof(input))
	  return(-1);
	for(k=0;k<c;k++,j++) {
	  ent=get_cmap_entry(bfptr->cmap,(int)c1);
	  if(set_entry(bfptr,i,j,ent)==-1)
	    return(-1);
	}
      }
                                  /*entering escape sequences*/
      else {   
	c=get_uchar(input);
	if(feof(input))
	  return(-1);
	if(c==0x00) {   /*end of line*/
	  j=0;
	  i--;
	}
        else if(c==0x01) return(0);  /*end of bitmap*/
	         /*next 2 bytes contain horizontal and vertical offset of */
	        /*next pixel*/
	else if(c==0x02) {  
	  c=get_uchar(input);
	  if(feof(input))
	    return(-1);
	  j+=c;
	  c=get_uchar(input);
	  if(feof(input))
	    return(-1);
	  i+=c;
	}
	                   /*entering absolute mode 0xff=> c => 0x03 */
	else {  
	  for(k=0;k<c;k++,j++) {
	    c1=get_uchar(input);
	    if(feof(input))
	      return(-1);
	    ent=get_cmap_entry(bfptr->cmap,(int)c1);
	    if(set_entry(bfptr,i,j,ent)==-1)
	      return(-1);
	  }
	  if(c & 1)     /* odd length run ,read pad byte*/ 
	    get_uchar(input);
	}
      }
    }
  }
  return(0);
}
/************************************************************************/
/*
 * This function releases the memory allocated when the bitmap is loaded.
 */
void release_bmp_file(BmpFilePtr bfptr)
{ 
  long height=bfptr->bfih.biheight;
  long width=bfptr->bfih.biwidth;
  int i;

  if (bfptr == NULL) return;

  if(bfptr->bfih.bibitcount!=24){   /*we have a color map , release it*/
    free(bfptr->cmap->table);
    free(bfptr->cmap);
  }
                             /*release bitmap data*/
  for(i=0;i<height;i++) 
    free(bfptr->image[i]);
  free(bfptr->image);
}
/************************************************************************/
/*
 * This function writes a bmp in 24 bit format.
 *  Parameters : fname - output file name.
 *               width  - image width.
 *               height - image height.
 *               img    - image data in r g b bytes array.
 * Return values: 0 Success.
 *                -1 Failure (failed opening output file).
 */
short write_bmp24(char *fname,long width,long height,unsigned char *img)
{
  FILE *output;

  if((output=fopen(fname,"wb"))==NULL) /*failed opening output file*/
    return(-1);
  if(write_bmp24_stream(output,width,height,img)==-1) {
    fclose(output);
    return(-1);
  }
  fclose(output);
  return(0);
}

/************************************************************************/
/*
 * This function writes a bmp in 24 bit format.
 *  Parameters : output - output stream.
 *               width  - image width.
 *               height - image height.
 *               img    - image data in r g b bytes array.
 * Return values: 0 Success.
 *                -1 Failure (failed opening output file).
 */
short write_bmp24_stream(FILE *output,long width,long height,
			 unsigned char *img)
{
  int i,j,padb;
              
  if((img==NULL) || (output==NULL) || (width<=0) || (height<=0))
    return(-1);
                               /*write the file header*/
  putc('B',output);
  putc('M',output);
         /*file size:  file header is 14                    */
        /*             file info is 40                     */
       /*              bitmap data is width*height*3 bytes*/
      /* No colormap.                                     */
 
  put_ulong(14+40+width*height*3,output);   
  put_ushort(0,output);
  put_ushort(0,output);
  put_ulong(54,output);

  
                  /*write info header*/

  put_ulong(40,output);   /*info header size*/
  put_long(width,output); /*image width*/
  put_long(height,output); /*image height*/
  put_ushort(1,output);  /*bit planes*/
  put_ushort(24,output); /*bit count*/
  put_ulong(BI_RGB,output);  /*compression*/
  put_ulong(width*height*3,output);  /*raw data size*/
  put_long(75*39,output);  /*75dpi*39 per meter*/
  put_long(75*39,output);  /*75dpi*39 per meter*/
  put_ulong(0,output);    /*colors used*/
  put_ulong(0,output);    /*colors important*/

       /*number of pad bytes at end of row so rows end on 32 bit boundry*/
  padb=(4-((width*3)%4)) & 0x03;

  for(i=height-1;i>=0;i--) {
    for(j=0;j<width;j++) {
      put_uchar(img[((i*width+j)*3)+2],output);
      put_uchar(img[((i*width+j)*3)+1],output);
      put_uchar(img[(i*width+j)*3],output);
    }
    for(j=0;j<padb;j++) 
     put_uchar('0',output);
  }
  return(0);
}

/************************************************************************/
long bmp_width(BmpFilePtr bfptr)
{
  return(bfptr->bfih.biwidth);
}
/************************************************************************/
long bmp_height(BmpFilePtr bfptr)
{
  return(bfptr->bfih.biheight);
}




