#include "copyright.h"

//=================================================================
// bmp_env.h - C++ envelope for portable 'bmp' file format.
//
// Description:
//
//
// RCS information:
// ----------------
// $RCSfile: bmp_env.h,v $
// $Revision: 1.6 $
// $Date: 1997/08/10 16:55:40 $
//
//=================================================================


#ifndef BMP_ENV_H
#define BMP_ENV_H

#include "image.h"

//extern "C" {
#include "bmp.h"
//}

class bmp {
  
  // The bmp struct is not set well for write - so we bypass it.
  
  unsigned char *data;
  unsigned width, height;
  
public:
  
  // I/O
  
  bmp &Read(char *fname);
  bmp &Read(FILE *fp);
  void Write(char *fname);
  void Write(FILE *fp);
  
  // Operators - from internal data type to bmp
  
  // mono image assign
  
  bmp &operator= (const fMonoImg &img) {
    width  = img.GetWidth();
    height = img.GetHeight();
    
    data = (unsigned char *) realloc(data, img.GetSize()*sizeof(RgbRec));
    
    unsigned char *p = data;
    for(register unsigned i=0; i<img.GetSize(); i++){
      *p++ = (unsigned char) (img.Pix(i) + 0.5);
      *p++ = (unsigned char) (img.Pix(i) + 0.5);
      *p++ = (unsigned char) (img.Pix(i) + 0.5);
    }
    return *this;
  }
  
  // color image assign
  
  bmp &operator= (const fRgbImg &img) {
    width  = img.GetWidth();
    height = img.GetHeight();
    
    data = (unsigned char *) realloc(data, img.GetSize()*sizeof(RgbRec));
    
    unsigned char *p = data;
    for(register unsigned i=0; i<img.GetSize(); i++){
      *p++ = (unsigned char) (img.Pix(i).Red() + 0.5);
      *p++ = (unsigned char) (img.Pix(i).Green() + 0.5);
      *p++ = (unsigned char) (img.Pix(i).Blue() + 0.5);
    }
    
    return *this;
  }
  
  
  // byte image assignment
  
  // byte mono image.
  
  bmp &operator= (const bMonoImg &img) {
    width  = img.GetWidth();
    height = img.GetHeight();
    
    data = (unsigned char *) realloc(data, img.GetSize()*sizeof(RgbRec));
    
    unsigned char *p = data;
    for(register unsigned i=0; i<img.GetSize(); i++){
      *p++ = (unsigned char) img.Pix(i) ;
      *p++ = (unsigned char) img.Pix(i) ;
      *p++ = (unsigned char) img.Pix(i) ;
    }
    
    return *this;
  }
  
  // byte color image.
  
  bmp &operator= (const bRgbImg &img) {
    width  = img.GetWidth();
    height = img.GetHeight();
    
    data = (unsigned char *) realloc(data, img.GetSize()*sizeof(RgbRec));
    
    unsigned char *p = data;
    for(register unsigned i=0; i<img.GetSize(); i++){
      *p++ = (unsigned char) img.Pix(i)[0];
      *p++ = (unsigned char) img.Pix(i)[1];
      *p++ = (unsigned char) img.Pix(i)[2];
    }
    
    return *this;
  }
  
  
  // casting operators - from all types of BMP to 
  // fMonoImg , fRgbImg, bMonoImg, bRgmImg, fCieImg.
  
  // ========= Float images ============

  // cast to float scale image.
  
  operator fMonoImg(){
    register unsigned i;
    
    fMonoImg tmp(width, height);

    unsigned char *p = data;
    for(i=0; i< width * height; i++){

	tmp.Pix(i) = 
	  (float) *p++ * RedToY   + // red
	  (float) *p++ * GreenToY + // green;
	  (float) *p++ * BlueToY;   // blue 
    }    
    return tmp;
  }
  
  
  // cast to color image
  
  operator fRgbImg(){

    register unsigned i;
    
    fRgbImg tmp(width,height);
    
    unsigned char *p = data; 
    for(i=0; i<width*height; i++){
	tmp.Pix(i).Red()    =  *p++;
	tmp.Pix(i).Green()  =  *p++;
	tmp.Pix(i).Blue()   =  *p++;
    }
    
    return tmp;
  }

  // cast to CIE color image (through fRgbPix).
  
  operator fCieImg(){

    register unsigned i;
    
    fCieImg tmp(width,height);
    
    unsigned char *p = data; 
    for(i=0; i<width*height; i++){
	unsigned char r = *p++;
	unsigned char g = *p++;
	unsigned char b = *p++;
	tmp.Pix(i) = fRgbPix(r,g,b);  // do not put *p++ here !
    }
    
    return tmp;
  }


  // ========= Byte images ============

  // cast to byte  image.
  
  operator bMonoImg(){
    register unsigned i;
    
    bMonoImg tmp(width, height);

    unsigned char *p = data;
    for(i=0; i< width * height; i++){
      
      tmp.Pix(i) = (unsigned char)
	((float) *p++ * RedToY   + // red
	 (float) *p++ * GreenToY + // green;
	 (float) *p++ * BlueToY);   // blue 
    }    
    return tmp;
  }
  
  
  // cast to color image
  
  operator bRgbImg(){

    register unsigned i;
    
    bRgbImg tmp(width,height);
    
    unsigned char *p = data; 
    for(i=0; i<width*height; i++){
      tmp.Pix(i).Red()    =  *p++;
      tmp.Pix(i).Green()  =  *p++;
      tmp.Pix(i).Blue()   =  *p++;
    }
    
    return tmp;
  }
  
  
  // constructors
  
  bmp(char *fname) { 
    Read(fname);
  }
  
  bmp(FILE *fp) { 
    Read(fp); 
  }
  
  // constructors - per supported output type !
  bmp(const fMonoImg &img){
    data = (unsigned char *) malloc(img.GetSize() * sizeof(RgbRec) );
    *this = img;
  }

  bmp(const fRgbImg &img){
    data = (unsigned char *) malloc(img.GetSize() * sizeof(RgbRec) );
    *this = img;
  }

  // byte images - used for exporting only.

  bmp(const bMonoImg &img){
    data = (unsigned char *) malloc(img.GetSize() * sizeof(RgbRec) );
    *this = img;
  }

  bmp(const bRgbImg &img){
    data = (unsigned char *) malloc(img.GetSize() * sizeof(RgbRec) );
    *this = img;
  }


  // distructor

  ~bmp() { 
    assert(data);
    free(data);
  }
};


#endif






