#include "copyright.h"

//===========================================================
// color.h - color pixel type template
//
// Description:
//
// color_t - Root color pixel template class.
//           All color space pixel definitions are derived 
//           from this class.
//
// RCS information:
// ----------------
// $RCSfile: color.h,v $
// $Revision: 1.3 $
// $Date: 1997/03/12 17:36:10 $
//
//===========================================================

#ifndef COLOR_H
#define COLOR_H
#include <stdlib.h>
#include <assert.h>

#ifndef NCHAN
#define NCHAN 3
#endif

template <class T> class color_t{

protected:
  T chan[NCHAN];
  
public:

  // Indexing operator
  T &operator[](const unsigned x) {
    assert (x < NCHAN);
    return chan[x];
  }

  const T &operator[](const unsigned x) const{
    assert (x < NCHAN);
    return chan[x];
  }

};


// RGB709 to CIE XYZ system
const float RedToX	= 0.412453F;
const float GreenToX	= 0.357580F;
const float BlueToX    	= 0.180423F;

const float RedToY     	= 0.212671F;
const float GreenToY	= 0.715160F;
const float BlueToY    	= 0.072169F;

const float RedToZ     	= 0.019334F;
const float GreenToZ	= 0.119193F;
const float BlueToZ    	= 0.950227F;


// CIE XYZ to RGB709 system
const float XtoRed     	=  3.240479F;
const float YtoRed     	= -1.537150F;
const float ZtoRed     	= -0.498535F;

const float XtoGreen	= -0.969256F;
const float YtoGreen	=  1.875992F;
const float ZtoGreen	=  0.041556F;

const float XtoBlue    	=  0.055648F;
const float YtoBlue    	= -0.204043F;
const float ZtoBlue    	=  1.057311F;
  
#endif







