#include "copyright.h"

//=================================================================
// image.cpp - image utilities.
//
// Description:
//	This file contains casting utilities for predefined image
//      types.
//
//
// RCS information:
// ----------------
// $RCSfile: image.cpp,v $
// $Revision: 1.1 $
// $Date: 2002/01/29 20:20:22 $
//
//=================================================================

#include "image.h"


//------------------------
// Float image conversions
//------------------------

// monochome image casting

fMonoImg::operator fRgbImg() {

  fRgbImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i];
  return tmp;
}


fMonoImg::operator bMonoImg() {
  bMonoImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = (unsigned char) (data[i] + 0.5);
  return tmp;
}

fMonoImg::operator bRgbImg() {
  bRgbImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++){
    tmp.Pix(i) = (unsigned char) (data[i] + 0.5);
  }
  return tmp;
}

fMonoImg::operator fCieImg() {
  fCieImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i];
  return tmp;
}

// rgb image casting  

fRgbImg::operator fMonoImg() {
  fMonoImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i].GetCieY();
  return tmp;
}


fRgbImg::operator bMonoImg() {
  bMonoImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = (unsigned char) (data[i].GetCieY() + 0.5);
  return tmp;
}

fRgbImg::operator bRgbImg() {
  bRgbImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++){
    tmp.Pix(i) = data[i];
  }
  return tmp;
}

fRgbImg::operator fCieImg() {
  fCieImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i];
  return tmp;
}


// Cie image casting  

fCieImg::operator fMonoImg() {
  fMonoImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i].CieY();
  return tmp;
}


fCieImg::operator bMonoImg() {
  bMonoImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = (unsigned char) (data[i].CieY() + 0.5);
  return tmp;
}


fCieImg::operator bRgbImg() {
  bRgbImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++){
    tmp.Pix(i) = data[i];
  }
  return tmp;
}


fCieImg::operator fRgbImg() {
  fRgbImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i];
  return tmp;
}




//------------------------
// unsigned char image conversions
//------------------------

// monochome image casting

bMonoImg::operator fRgbImg() {
  fRgbImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i];
  return tmp;
}


bMonoImg::operator fMonoImg() {
  fMonoImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i];
  return tmp;
}

bMonoImg::operator bRgbImg() {
  bRgbImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++){
    tmp.Pix(i) = data[i];
  }
  return tmp;
}

bMonoImg::operator fCieImg() {
  fCieImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i];
  return tmp;
}

// rgb image casting  

bRgbImg::operator fMonoImg() {
  fMonoImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i].GetCieY();
  return tmp;
}


bRgbImg::operator bMonoImg() {
  bMonoImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = (unsigned char) (data[i].GetCieY() + 0.5) ;
  return tmp;
}

bRgbImg::operator fRgbImg() {
  fRgbImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++){
    tmp.Pix(i) = data[i];
  }
  return tmp;
}

bRgbImg::operator fCieImg() {
  fCieImg tmp(width, height);
  for(register unsigned i=0; i < GetSize(); i++)
    tmp.Pix(i) = data[i];
  return tmp;
}



//----------------------
// define common kernels
//----------------------

// derivatives
const float d3[3] = { (float) 0.5,    0.0F, -0.5F                                  };
const float d5[5] = { 0.25F,   0.5F,  0.0F,   -0.5F, -0.25F                   };
const float d7[7] = { 0.0625F, 0.25F, 0.3125F, 0.0F, -0.3125F, -0.25F, -0.0625F };

fKernel Di2Dx3(3,1,d3);
fKernel Di2Dy3(1,3,d3);
fKernel Di2Dx5(5,1,d5);
fKernel Di2Dy5(1,5,d5);
fKernel Di2Dx7(7,1,d7);
fKernel Di2Dy7(1,7,d7);

// gousian (seperable). 
const float g3[3] = { 0.25F,   0.5F,   0.25F                     };
const float g5[5] = { 0.0625F, 0.25F,   0.375F, 0.25F,  0.0625F  };
const float g7[7] = { 0.0156F, 0.0938F, 0.2344F, 0.3125F, 
                      0.2344F, 0.0938F, 0.0156F                  };

fKernel Gx3(3,1,g3);
fKernel Gy3(1,3,g3);
fKernel Gx5(5,1,g5);
fKernel Gy5(1,5,g5);
fKernel Gx7(7,1,g7);
fKernel Gy7(1,7,g7);



