#include "copyright.h"

//=================================================================
// image.h - Incore image definition.
//
// Description:
// 	This file contains predfrined image types and operators
//	The types are subclasses not typedefs to allow extentions
//      and modifications for specific types.
//
//
// RCS information:
// ----------------
// $RCSfile: image.h,v $
// $Revision: 1.5 $
// $Date: 1997/08/10 16:55:40 $
//
//=================================================================

#ifndef IMAGE_H
#define IMAGE_H

#include <stdio.h>

#include "rgb.h"
#include "cie.h"
#include "img_op.h"


//-------------------------
// define actual data types
//-------------------------

class fMonoImg ;
class fRgbImg ;

class bMonoImg;
class bRgbImg;

class fCieImg;

class fKernel;

// some predefinded kefnels.
extern fKernel Di2Dx3;
extern fKernel Di2Dy3;
extern fKernel Di2Dx5;
extern fKernel Di2Dy5;
extern fKernel Di2Dx7;
extern fKernel Di2Dy7;
extern fKernel Gx3;
extern fKernel Gy3;
extern fKernel Gx5;
extern fKernel Gy5;
extern fKernel Gx7;
extern fKernel Gy7;
//---------------------------


//========================================================
// float classes are the incore classes all operators 
// are defined on float classes.
//========================================================


class fMonoImg : public img_t<float> {

public:
  
  // casting
  operator fRgbImg(); 
  operator bMonoImg();
  operator bRgbImg();
  operator fCieImg();
  
  // operators
  fMonoImg &operator=(const float val){
    img_t<float>::operator=(val);
    return *this;
  }
  fMonoImg &operator+=(const float val){
    img_t<float>::operator+=(val);
    return *this;
  }
  fMonoImg &operator-=(const float val){
    img_t<float>::operator-=(val);
    return *this;
  }
  fMonoImg &operator*=(const float val){	  
    img_t<float>::operator*=(val);
    return *this;
  }
  fMonoImg &operator/=(const float val){
    img_t<float>::operator/=(val);
    return *this;
  }
  fMonoImg operator+(const float val){
    return (fMonoImg) img_t<float>::operator+(val);
  }
  fMonoImg operator-(const float val){
    return (fMonoImg) img_t<float>::operator-(val);
  }
  fMonoImg operator*(const float val){
    return (fMonoImg) img_t<float>::operator*(val);
  }
  fMonoImg operator/(const float val){
    return (fMonoImg) img_t<float>::operator/(val);
  }
  fMonoImg &operator=(const fMonoImg &img){
    img_t<float>::operator=(img);
    return *this;
  }
  fMonoImg &operator+=(const fMonoImg &img){
    img_t<float>::operator+=(img);
    return *this;
  }
  fMonoImg &operator-=(const fMonoImg &img){
    img_t<float>::operator-=(img);
    return *this;
  }
  fMonoImg &operator*=(const fMonoImg &img){
    img_t<float>::operator*=(img);
    return *this;
  }
  fMonoImg &operator/=(const fMonoImg &img){
    img_t<float>::operator/=(img);
    return *this;
  }
  fMonoImg operator+(const fMonoImg &img){
    return (fMonoImg) img_t<float>::operator+(img);
  }
  fMonoImg operator-(const fMonoImg &img){
    return (fMonoImg) img_t<float>::operator-(img);
  }
  fMonoImg operator*(const fMonoImg &img){
    return (fMonoImg) img_t<float>::operator*(img);
  }
  fMonoImg operator/(const fMonoImg &img){
    return (fMonoImg) img_t<float>::operator/(img);
  }
  fMonoImg operator*(fKernel &ker){
    return (fMonoImg) img_t<float>::operator*(ker);
  }
  fMonoImg operator-(){
    return (fMonoImg) img_t<float>::operator-();
  }

  fMonoImg map(float f(float x)){
    return (fMonoImg) img_t<float>::map(f);
  }
  
  fMonoImg map(float f(int x, int y, float v)){
    return (fMonoImg) img_t<float>::map(f);
  }


  
  // contructor
  fMonoImg() : img_t<float>(){};
  fMonoImg(const unsigned w, const unsigned h, 
	   const int ox=0,   const int oy=0) : img_t<float>(w, h,ox,oy){};
  fMonoImg(const img_t<float> &img) : img_t<float>(img){};
};


class fRgbImg : public img_t<fRgbPix>{

public:

  //operatores
	
  // casting
  operator fMonoImg() ;
  operator bMonoImg();
  operator bRgbImg() ;
  operator fCieImg() ;
  
  // operators
  fRgbImg &operator=(const fRgbPix &val){
    img_t<fRgbPix>::operator=(val);
    return *this;
  }

  fRgbImg &operator+=(const fRgbPix &val){
    img_t<fRgbPix>::operator+=(val);
    return *this;
  }
  fRgbImg &operator-=(const fRgbPix &val){
    img_t<fRgbPix>::operator-=(val);
    return *this;
  }

  fRgbImg &operator*=(const fRgbPix &val){	  
    img_t<fRgbPix>::operator*=(val);
    return *this;
  }
  fRgbImg &operator/=(const fRgbPix &val){
    img_t<fRgbPix>::operator/=(val);
    return *this;
  }
  fRgbImg operator+(const fRgbPix &val){
    return (fRgbImg) img_t<fRgbPix>::operator+(val);
  }
  fRgbImg operator-(const fRgbPix &val){
    return (fRgbImg) img_t<fRgbPix>::operator-(val);
  }
  fRgbImg operator*(const fRgbPix &val){
    return (fRgbImg) img_t<fRgbPix>::operator*(val);
  }
  fRgbImg operator/(const fRgbPix &val){
    return (fRgbImg) img_t<fRgbPix>::operator/(val);
  }
  fRgbImg &operator=(const fRgbImg &img){
    img_t<fRgbPix>::operator=(img);
    return *this;
  }
  fRgbImg &operator+=(const fRgbImg &img){
    img_t<fRgbPix>::operator+=(img);
    return *this;
  }
  fRgbImg &operator-=(const fRgbImg &img){
    img_t<fRgbPix>::operator-=(img);
    return *this;
  }
  fRgbImg &operator*=(const fRgbImg &img){
    img_t<fRgbPix>::operator*=(img);
    return *this;
  }
  fRgbImg &operator/=(const fRgbImg &img){
    img_t<fRgbPix>::operator/=(img);
    return *this;
  }
  fRgbImg operator+(const fRgbImg &img){
    return (fRgbImg) img_t<fRgbPix>::operator+(img);
  }
  fRgbImg operator-(const fRgbImg &img){
    return (fRgbImg) img_t<fRgbPix>::operator-(img);
  }
  fRgbImg operator*(const fRgbImg &img){
    return (fRgbImg) img_t<fRgbPix>::operator*(img);
  }
  fRgbImg operator/(const fRgbImg &img){
    return (fRgbImg) img_t<fRgbPix>::operator/(img);
  }
  fRgbImg operator*(fKernel &ker){
    return (fRgbImg) img_t<fRgbPix>::operator*(ker);
  }

  fRgbImg operator-(){
    return (fRgbImg) img_t<fRgbPix>::operator-();
  }


  fRgbImg map(fRgbPix f(fRgbPix x)){
    return (fRgbImg) img_t<fRgbPix>::map(f);
  }
  
  fRgbImg map(fRgbPix f(int x, int y, fRgbPix v)){
    return (fRgbImg) img_t<fRgbPix>::map(f);
  }


  
  // constructors
  fRgbImg() : img_t<fRgbPix>(){};
  fRgbImg(const unsigned w, const unsigned h, 
	  const int ox=0,   const int oy=0) : img_t<fRgbPix>(w, h,ox,oy){};
  
  // copy constructor
  fRgbImg(const fRgbImg &img) : img_t<fRgbPix>(img){};
  fRgbImg(const img_t<fRgbPix> &img) : img_t<fRgbPix>(img){};
  
};



class fCieImg : public img_t<fCiePix>{
  
public:
  
  //operatores
  
  // casting
  operator fMonoImg() ;
  operator bMonoImg();
  operator fRgbImg() ;
  operator bRgbImg();
  
  // operators
  fCieImg &operator=(const fCiePix &val){
    img_t<fCiePix>::operator=(val);
    return *this;
  }
  fCieImg &operator+=(const fCiePix &val){
    img_t<fCiePix>::operator+=(val);
    return *this;
  }
  fCieImg &operator-=(const fCiePix &val){
    img_t<fCiePix>::operator-=(val);
    return *this;
  }
  fCieImg &operator*=(const fCiePix &val){	  
    img_t<fCiePix>::operator*=(val);
    return *this;
  }
  fCieImg &operator/=(const fCiePix &val){
    img_t<fCiePix>::operator/=(val);
    return *this;
  }
  fCieImg operator+(const fCiePix &val){
    return (fCieImg) img_t<fCiePix>::operator+(val);
  }
  fCieImg operator-(const fCiePix &val){
    return (fCieImg) img_t<fCiePix>::operator-(val);
  }
  fCieImg operator*(const fCiePix &val){
    return (fCieImg) img_t<fCiePix>::operator*(val);
  }
  fCieImg operator/(const fCiePix &val){
    return (fCieImg) img_t<fCiePix>::operator/(val);
  }
  fCieImg &operator=(const fCieImg &img){
    img_t<fCiePix>::operator=(img);
    return *this;
  }
  fCieImg &operator+=(const fCieImg &img){
    img_t<fCiePix>::operator+=(img);
    return *this;
  }
  fCieImg &operator-=(const fCieImg &img){
    img_t<fCiePix>::operator-=(img);
    return *this;
  }
  fCieImg &operator*=(const fCieImg &img){
    img_t<fCiePix>::operator*=(img);
    return *this;
  }
  fCieImg &operator/=(const fCieImg &img){
    img_t<fCiePix>::operator/=(img);
    return *this;
  }
  fCieImg operator+(const fCieImg &img){
    return (fCieImg) img_t<fCiePix>::operator+(img);
  }
  fCieImg operator-(const fCieImg &img){
    return (fCieImg) img_t<fCiePix>::operator-(img);
  }
  fCieImg operator*(const fCieImg &img){
    return (fCieImg) img_t<fCiePix>::operator*(img);
  }
  fCieImg operator/(const fCieImg &img){
    return (fCieImg) img_t<fCiePix>::operator/(img);
  }

  fCieImg operator*(fKernel &ker){
    return (fCieImg) img_t<fCiePix>::operator*(ker);
  }
  
  fCieImg operator-(){
    return (fCieImg) img_t<fCiePix>::operator-();
  }
  
  // constructors
  fCieImg() : img_t<fCiePix>(){};
  fCieImg(const unsigned w, const unsigned h, 
	  const int ox=0,   const int oy=0) : img_t<fCiePix>(w, h,ox,oy){};
  // copy constructor
  fCieImg(const fCieImg &img) : img_t<fCiePix>(img){};
  fCieImg(const img_t<fCiePix> &img) : img_t<fCiePix>(img){};
  
};


//========================================================
// unsigned char classes are used for efficeint operations and IO 
// Only gray scale and rgb are supported. No arrithmetic 
// operators suppoted.
//========================================================

class bMonoImg : public image_t<unsigned char> {
  
public:
  
  // casting
  operator fRgbImg(); 
  operator fMonoImg();
  operator bRgbImg();
  operator fCieImg();
  
  // operators
  bMonoImg &operator=(const unsigned char val){
    image_t<unsigned char>::operator=(val);
    return *this;
  }

  // contructor
  bMonoImg() : image_t<unsigned char>(){};
  bMonoImg(const unsigned w, const unsigned h, 
	   const int ox=0,   const int oy=0) : image_t<unsigned char>(w, h,ox,oy){};
  bMonoImg(const image_t<unsigned char> &img) : image_t<unsigned char>(img){};
};


class bRgbImg : public image_t<bRgbPix>{

public:

  //operatores
	
  // casting
  operator fMonoImg() ;
  operator bMonoImg();
  operator fRgbImg() ;
  operator fCieImg() ;
  
  // operators
  bRgbImg &operator=(const bRgbPix &val){
    image_t<bRgbPix>::operator=(val);
    return *this;
  }

  
  // constructors
  bRgbImg() : image_t<bRgbPix>(){};
  bRgbImg(const unsigned w, const unsigned h, 
	  const int ox=0,   const int oy=0) : image_t<bRgbPix>(w, h,ox,oy){};
  
  // copy constructor
  bRgbImg(const bRgbImg &img) : image_t<bRgbPix>(img){};
  bRgbImg(const image_t<bRgbPix> &img) : image_t<bRgbPix>(img){};
  
};

#endif

