#include "copyright.h"
#include "error_msg.h"

/*=================================================================*
 * pnm.c - pnm rutines
 *
 * Description:
 *   Creat, Free, Read, Write PNM raw file formats (pgm, ppm) 
 *   and private extention for the PNM family. 
 *
 * RCS information:
 * ----------------
 * $RCSfile: pnm.cpp,v $
 * $Revision: 1.1 $
 * $Date: 2002/01/29 20:20:22 $
 *
 *=================================================================*/


#include <stdio.h>
#include <stdlib.h>
#include "pnm.h"
static char comment[1024];


/*
 * create_image - create new image according to input parameters
 */

pnm_t *create_image(int width, int height, int type)
{
  static pnm_t *newimg;

  if ((newimg = (pnm_t *) malloc(sizeof (pnm_t))) == NULL){
	error_msg("pnm: create_image - malloc failed",0);
	clip_exception(MALLOC);
  }

  newimg->type = type;
  newimg->width = width;
  newimg->height = height;
  newimg->maxval = 255;

  if ((newimg->data = (unsigned char *) malloc(width * height * PSIZE(type))) == NULL){
	error_msg("pnm: create_image - malloc failed",0);
	clip_exception(MALLOC);
  }

  return newimg;
}


/*
 * free image
 */

void free_image(pnm_t *img)
{
	if (img->data != NULL)
		free(img->data);
	free(img);
}


/*
 * read_image -  read pnm format image
 */

pnm_t *read_image(FILE *fp)
{
  static pnm_t *image;

  if ((image = (pnm_t *) malloc(sizeof(pnm_t))) == NULL){
	error_msg("pnm: read_image - malloc failed",0);
	clip_exception(MALLOC);
  }


  if (fscanf(fp,"P%d\n", &image->type) !=1) {
    error_msg("Pnm: rw: Not A pnm file",0);
	clip_exception(UNKNOWN_FORMAT);
  }

  if (PSIZE(image->type) <= 0 ) {
    error_msg("Pnm: rw: Pnm type %d - not supported",image->type);
	clip_exception(UNKNOWN_FORMAT);
  }
    

  while(fscanf(fp,"#%[^\n]\n",comment) > 0);

  fscanf(fp,"%u%u\n", &image->width, &image->height);
  fscanf(fp,"%u%*c",   &image->maxval);   // force the new-line to be one char

  if((image->data = (unsigned char *)
        malloc(image->width * image->height * PSIZE(image->type))) == NULL){
	error_msg("pnm: read_image - malloc failed",0);
	clip_exception(MALLOC);
  }

  fread(image->data,
        image->width * image->height,
        PSIZE(image->type),
        fp);

  return image;
}


/*
 * write-image - write pnm format image
 */

void write_image(FILE *fp, pnm_t *image)
{
  fprintf(fp,"P%d\n", image->type);
  fprintf(fp,"%u %u\n", image->width, image->height);
  fprintf(fp,"%u%c", image->maxval,10);   // force a ^J newline

  fwrite(image->data,
         image->width * image->height,
         PSIZE(image->type),
         fp);
}


