#include "copyright.h"

//============================================================
// rgb.h - rgb pixel type class
//
// Description:
//
// fRgbPix  - rgb color space pixel class
//
// RCS information:
// ----------------
// $RCSfile: rgb.h,v $
// $Revision: 1.6 $
// $Date: 1997/04/02 17:55:26 $
//
//============================================================

#ifndef RGB_H
#define RGB_H

#include "color.h"

const unsigned char RED   = 0;
const unsigned char GREEN = 1;
const unsigned char BLUE  = 2;

class fCiePix;
class bRgbPix;

class fRgbPix : public color_t<float>{

public:
  float &Red() {return operator[](RED);}
  const float &Red() const {return operator[](RED);}
  float &Green() {return operator[] (GREEN);}
  const float &Green() const {return operator[](GREEN);}
  float &Blue() {return operator[] (BLUE);}
  const float &Blue() const {return operator[](BLUE);}

  float GetCieX() const { 
    return	Red()   * RedToX   + 
		Green() * GreenToX + 
		Blue()  * BlueToX; 
  }
  float GetCieY() const { 
    return	Red()   * RedToY   + 
		Green() * GreenToY + 
		Blue()  * BlueToY; 
  }
  float GetCieZ() const { 
    return	Red()   * RedToZ   + 
		Green() * GreenToZ + 
		Blue()  * BlueToZ; 
  }

 
  // assign and cast operators.

  fRgbPix &operator=(const float v){	// float level
    Red()   = v;
    Green() = v;
    Blue()  = v;
    return *this;
  }

  fRgbPix &operator=(const fRgbPix &pix){	
    Red()   = pix.Red();
    Green() = pix.Green();
    Blue()  = pix.Blue();
    return *this;
  }


  fRgbPix &operator=(const bRgbPix &pix);
  fRgbPix &operator=(const fCiePix &pix);


  // casting operators
  operator fCiePix();
  operator bRgbPix();

  // Pixel Constant reflexive operators

  fRgbPix &operator+= (const float c){
	Red()   += c;
	Green() += c;
	Blue()  += c;
	return *this;
  }

  fRgbPix &operator-= (const float c){
	Red()   -= c;
	Green() -= c;
	Blue()  -= c;
	return *this;
  }

  fRgbPix &operator*= (const float c){
	Red()   *= c;
	Green() *= c;
	Blue()  *= c;
	return *this;
  }

  fRgbPix &operator/= (const float c){
	Red()   /= c;
	Green() /= c;
	Blue()  /= c;
	return *this;
  }

  // Pixel Constant non-reflexive operators

  fRgbPix operator+ (const float c){
	fRgbPix tmp;
	tmp.Red()   = Red()   + c;
	tmp.Green() = Green() + c;
	tmp.Blue()  = Blue()  + c;
	return tmp;
  }

  fRgbPix operator- (const float c){
	fRgbPix tmp;
	tmp.Red()   = Red()   - c;
	tmp.Green() = Green() - c;
	tmp.Blue()  = Blue()  - c;
	return tmp;
  }

  fRgbPix operator* (const float c){
	fRgbPix tmp;
	tmp.Red()   = Red()   * c;
	tmp.Green() = Green() * c;
	tmp.Blue()  = Blue()  * c;
	return tmp;
  }

  fRgbPix operator/ (const float c){
	fRgbPix tmp;
	tmp.Red()   = Red()   / c;
	tmp.Green() = Green() / c;
	tmp.Blue()  = Blue()  / c;
	return tmp;
  }


  // Pixel Pixel reflexive operators

  fRgbPix &operator+= (const fRgbPix &pix){
	Red()   += pix.Red();
	Green() += pix.Green();
	Blue()  += pix.Blue();
	return *this;
  }

  fRgbPix &operator-= (const fRgbPix &pix){
	Red()   -= pix.Red();
	Green() -= pix.Green();
	Blue()  -= pix.Blue();
	return *this;
  }

  fRgbPix &operator*= (const fRgbPix &pix){
	Red()   *= pix.Red();
	Green() *= pix.Green();
	Blue()  *= pix.Blue();
	return *this;
  }

  fRgbPix &operator/= (const fRgbPix &pix){
	Red()   /= pix.Red();
	Green() /= pix.Green();
	Blue()  /= pix.Blue();
	return *this;
  }

  // Pixel Pixel non-reflexive operators

  fRgbPix operator+ (const fRgbPix &pix){
	fRgbPix tmp;
	tmp.Red()   = Red()   + pix.Red();
	tmp.Green() = Green() + pix.Green();
	tmp.Blue()  = Blue()  + pix.Blue();
	return tmp;
  }

  fRgbPix operator- (const fRgbPix &pix){
	fRgbPix tmp;
	tmp.Red()   = Red()   - pix.Red();
	tmp.Green() = Green() - pix.Green();
	tmp.Blue()  = Blue()  - pix.Blue();
	return tmp;
  }

  fRgbPix operator* (const fRgbPix &pix){
	fRgbPix tmp;
	tmp.Red()   = Red()   * pix.Red();
	tmp.Green() = Green() * pix.Green();
	tmp.Blue()  = Blue()  * pix.Blue();
	return tmp;
  }

  fRgbPix operator/ (const fRgbPix &pix){
	fRgbPix tmp;
	tmp.Red()   = Red()   / pix.Red();
	tmp.Green() = Green() / pix.Green();
	tmp.Blue()  = Blue()  / pix.Blue();
	return tmp;
  }

  
  // constructors
  fRgbPix() {};

  fRgbPix(const float v){
	Red() = Green() = Blue() = v;
  }

  fRgbPix(const float r, const float g, const float b) {
    Red()   = r;
    Green() = g;
    Blue()  = b;
  }

  // copy constructor

  fRgbPix(const fRgbPix &pix){
    Red()   = pix.Red();
    Green() = pix.Green();
    Blue()  = pix.Blue();
  }
 

 
};


//=========================================
// Brgb is not template - to allow casting
// and to diffeenciate in operation sematics. 
//=========================================


class bRgbPix : public color_t<unsigned char>{

public:
  unsigned char &Red() {return operator[](RED);}
  const unsigned char &Red() const {return operator[](RED);}
  unsigned char &Green() {return operator[] (GREEN);}
  const unsigned char &Green() const {return operator[](GREEN);}
  unsigned char &Blue() {return operator[] (BLUE);}
  const unsigned char &Blue() const {return operator[](BLUE);}

  float GetCieX() const { 
    return	(float) Red()   * RedToX   + 
		(float) Green() * GreenToX + 
		(float) Blue()  * BlueToX; 
  }
  float GetCieY() const { 
    return	(float) Red()   * RedToY   + 
		(float) Green() * GreenToY + 
		(float) Blue()  * BlueToY; 
  }
  float GetCieZ() const { 
    return	(float) Red()   * RedToZ   + 
		(float) Green() * GreenToZ + 
		(float) Blue()  * BlueToY; 
  }

 
  // assign and cast operators.

  bRgbPix &operator=(const unsigned char v){	// unsigned char level
    Red()   = v;
    Green() = v;
    Blue()  = v;
    return *this;
  }

  bRgbPix &operator=(const bRgbPix &pix){	
    Red()   = pix.Red();
    Green() = pix.Green();
    Blue()  = pix.Blue();
    return *this;
  }


  bRgbPix &operator=(const fRgbPix &pix);
  bRgbPix &operator=(const fCiePix &pix);


  // casting operators
  operator fCiePix();
  operator fRgbPix();

  // Pixel Constant reflexive operators

  bRgbPix &operator+= (const unsigned char c){ 
	Red()   += c;
	Green() += c;
	Blue()  += c;
	return *this;
  }

  bRgbPix &operator-= (const unsigned char c){
	Red()   -= c;
	Green() -= c;
	Blue()  -= c;
	return *this;
  }

  bRgbPix &operator*= (const unsigned char c){
	Red()   *= c;
	Green() *= c;
	Blue()  *= c;
	return *this;
  }

  bRgbPix &operator/= (const unsigned char c){
	Red()   /= c;
	Green() /= c;
	Blue()  /= c;
	return *this;
  }

  // Pixel Constant non-reflexive operators

  bRgbPix operator+ (const unsigned char c){
	bRgbPix tmp;
	tmp.Red()   = Red()   + c;
	tmp.Green() = Green() + c;
	tmp.Blue()  = Blue()  + c;
	return tmp;
  }

  bRgbPix operator- (const unsigned char c){
	bRgbPix tmp;
	tmp.Red()   = Red()   - c;
	tmp.Green() = Green() - c;
	tmp.Blue()  = Blue()  - c;
	return tmp;
  }

  bRgbPix operator* (const unsigned char c){
	bRgbPix tmp;
	tmp.Red()   = Red()   * c;
	tmp.Green() = Green() * c;
	tmp.Blue()  = Blue()  * c;
	return tmp;
  }

  bRgbPix operator/ (const unsigned char c){
	bRgbPix tmp;
	tmp.Red()   = Red()   / c;
	tmp.Green() = Green() / c;
	tmp.Blue()  = Blue()  / c;
	return tmp;
  }


  // Pixel Pixel reflexive operators

  bRgbPix &operator+= (const bRgbPix &pix){
	Red()   += pix.Red();
	Green() += pix.Green();
	Blue()  += pix.Blue();
	return *this;
  }

  bRgbPix &operator-= (const bRgbPix &pix){
	Red()   -= pix.Red();
	Green() -= pix.Green();
	Blue()  -= pix.Blue();
	return *this;
  }

  bRgbPix &operator*= (const bRgbPix &pix){
	Red()   *= pix.Red();
	Green() *= pix.Green();
	Blue()  *= pix.Blue();
	return *this;
  }

  bRgbPix &operator/= (const bRgbPix &pix){
	Red()   /= pix.Red();
	Green() /= pix.Green();
	Blue()  /= pix.Blue();
	return *this;
  }

  // Pixel Pixel non-reflexive operators

  bRgbPix operator+ (const bRgbPix &pix){
	bRgbPix tmp;
	tmp.Red()   = Red()   + pix.Red();
	tmp.Green() = Green() + pix.Green();
	tmp.Blue()  = Blue()  + pix.Blue();
	return tmp;
  }

  bRgbPix operator- (const bRgbPix &pix){
	bRgbPix tmp;
	tmp.Red()   = Red()   - pix.Red();
	tmp.Green() = Green() - pix.Green();
	tmp.Blue()  = Blue()  - pix.Blue();
	return tmp;
  }

  bRgbPix operator* (const bRgbPix &pix){
	bRgbPix tmp;
	tmp.Red()   = Red()   * pix.Red();
	tmp.Green() = Green() * pix.Green();
	tmp.Blue()  = Blue()  * pix.Blue();
	return tmp;
  }

  bRgbPix operator/ (const bRgbPix &pix){
	bRgbPix tmp;
	tmp.Red()   = Red()   / pix.Red();
	tmp.Green() = Green() / pix.Green();
	tmp.Blue()  = Blue()  / pix.Blue();
	return tmp;
  }

  
  // constructors
  bRgbPix() {};

  bRgbPix(const unsigned char v){
	Red() = Green() = Blue() = v;
  }

  bRgbPix(const unsigned char r, const unsigned char g, const unsigned char b) {
    Red()   = r;
    Green() = g;
    Blue()  = b;
  }

  // copy constructor

  bRgbPix(const bRgbPix &pix){
    Red()   = pix.Red();
    Green() = pix.Green();
    Blue()  = pix.Blue();
  }
};

#endif






